# -*- encoding: utf-8 -*-

import os
import signal
import subprocess
from zerocs.fork import Fork
from zerocs.interface import Interface
from zerocs.build import ServiceBuild, WorkBuild
from zerocs.utils import Utils

current_directory = os.path.dirname(os.path.abspath(__file__))


def print_exception(e):
    pass


@Fork
class _Fork:

    @staticmethod
    def fork_logger(config: dict) -> None:
        _python = Interface.get_python_path()
        _script = os.path.join(current_directory, 'fork_logger.py')

        cmd = f'{_python} {_script} ' \
              f'--RABBITMQ_CONFIG {config.get("RABBITMQ_CONFIG")} ' \
              f'--LOGS_PATH {config.get("LOGS_PATH")} ' \
              f'--SERVICE_IP {Utils.get_ipaddr()}'

        if os.name == 'nt':
            subprocess.Popen(cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
        else:
            subprocess.Popen(cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)

    @staticmethod
    def fork_service(service_list: list, config: dict) -> None:
        for service in service_list:
            func = ServiceBuild.build(func=service.RpcFunction, rabbitmq_config=config.get('RABBITMQ_CONFIG'))
            _python = Interface.get_python_path()
            _script = os.path.join(current_directory, 'fork_service.py')

            _apis = func.__dict__.get('apis')
            _service_id = func.__dict__.get('service_id')
            _service_ip = func.__dict__.get('service_ip')
            _service_name = func.__dict__.get('service_name')

            _server = Interface.get_service_by_name_and_ip(service_name=_service_name, service_ip=_service_ip)
            _service_pid = _server.get("service_pid")

            try:
                os.kill(_service_pid, signal.SIGTERM)
            except Exception as e:
                print_exception(e)

            cmd = f'{_python} {_script} ' \
                  f'--SERVICE_PATH {service.__name__} ' \
                  f'--RABBITMQ_CONFIG {config.get("RABBITMQ_CONFIG")} ' \
                  f'--LOGS_PATH {config.get("LOGS_PATH")}'

            if os.name == 'nt':
                process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
            else:
                process = subprocess.Popen(
                    cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)

            Interface.insert_service(
                api_list=_apis,
                service_id=_service_id,
                service_ip=_service_ip,
                max_work=1, run_work=[],
                service_pid=process.pid,
                service_name=_service_name,
                update_time=Interface.get_time_str('%Y-%m-%d %H:%M:%S', 'Asia/Shanghai'),
            )

    @staticmethod
    def fork_work(work_list: list, config: dict) -> None:
        for work in work_list:
            func = WorkBuild.build(func=work.WorkFunction, rabbitmq_config=config.get('RABBITMQ_CONFIG'))
            _python = Interface.get_python_path()
            _script = os.path.join(current_directory, 'fork_work.py')

            _work_id = func.__dict__.get('work_id')
            _work_ip = func.__dict__.get('work_ip')
            _work_name = func.__dict__.get('work_name')

            _work = Interface.get_service_by_name_and_ip(service_name=_work_name, service_ip=_work_ip)
            _work_pid = _work.get("work_pid")
            try:
                os.kill(_work_pid, signal.SIGTERM)
            except Exception as e:
                print_exception(e)

            cmd = f'{_python} {_script} ' \
                  f'--SERVICE_PATH {work.__name__} ' \
                  f'--RABBITMQ_CONFIG {config.get("RABBITMQ_CONFIG")} ' \
                  f'--LOGS_PATH {config.get("LOGS_PATH")} ' \
                  f'--MONGODB_CONFIG {config.get("MONGODB_CONFIG")}'

            if os.name == 'nt':
                process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE, stdout=subprocess.PIPE)
            else:
                process = subprocess.Popen(
                    cmd, stdin=subprocess.PIPE, stderr=subprocess.PIPE, stdout=subprocess.PIPE, shell=True)

            data = {"work_id": _work_id, "work_pid": process.pid}
            Interface.update_service_by_name_and_ip(service_name=_work_name, service_ip=_work_ip, data=data)
